*     JLdL 02Jan21.
*     
*     Copyright (C) 2020-2021 by Jorge L. deLyra <delyra@lmail.if.usp.br>.
*     This program can be copied and/or distributed freely under the
*     GPL license; see the terms and conditions in the files within
*     the sub-directory doc/ of this source directory.
*     
*     A program to plot the variable \xi_{M} associated to the total
*     energy of the bound system, as a function of the inner radial
*     variable \xi_{1}, in order to look for the most stable solution,
*     that is, the solution with the lowest total energy, under the
*     constraint that the total energy of the corresponding unbound
*     system remains constant; this routine uses the following
*     additional local double-precision temporary variables:
*     
*     xi_del_var   : local temporary for the plotting variation;
*     xienergy_var : local temporary for the energy.
*     
*     Considering the initial bounds xim_var_1 and xim_var_2 of
*     \xi_{M}, needed in order to determine the value of \xi_{M}
*     by the exponential sandwich method, note that we have the
*     following limits for the maximum and minimum possible values
*     of \xi_{M}; first, since this energy must be positive,
*     
*     xim_var_1 = \xi_{M,1} (lower bound): \xi_{M} > 0;
*     
*     second, since the total energy of the bound system must be
*     smaller than the energy of the unbound suystem,
*     
*     xim_var_2 = \xi_{M,2} (upper bound): \xi_{M} < {\cal V}/3;
*     
*     this last condition is equivalent to the statement that the
*     radial constant r_{\mu} > 0 is strictly positive.
*     
*     Complex formulas contained in this routine: none.
*     
      PROGRAM XIENERGY
*     
*     Double precision everything; due to the gfortran compiler option
*     -fdefault-real-8, which is in use in the Makefile, this is actually
*     quadruple precision, in the traditional sense of the term, that is,
*     it is actually real*16 (128-bit real data).
      implicit double precision (a-h,o-z)
*     
*     Include the global fixed parameters.
      include 'fp_def.f'
*     
*     Dimension three multicolumn plotting arrays.
      dimension xi1variable(0:mp_dat)
      dimension xi2variable(0:mp_dat)
      dimension boundenergy(0:mp_dat)
*     
*     Acquire the basic number of integration points.
 100  write(*,'(1a,$)')
     *     'Enter the number of integration points (ni_dat>0) [1000]:  '
      read(*,'(i43)',err=100) ni_dat
*     
*     Implement the default.
      if (ni_dat.le.0) then
         write(*,'(1a)')
     *        '        Adopting the value 1000 for ni_dat.'
         ni_dat=1000
      end if
*     
*     Acquire the number of plotting points.
 200  write(*,'(1a,$)')
     *     'Enter the number of plotting points (np_dat>0) [40]:       '
      read(*,'(i43)',err=200) np_dat
*     
*     Implement the default.
      if (np_dat.le.0) then
         write(*,'(1a)')
     *        '        Adopting the value 40 for np_dat.'
         np_dat=40
      end if
*     
*     Acquire the value of the dimensionless coordinate volume.
 300  write(*,'(1a,$)')
     *     'Enter the value of the coordinate volume (>0.0) [5.0]:     '
      read(*,'(f43.36)',err=300) fix_vol
*     
*     Prevention against a very small non-zero value.
      if ( (fix_vol.gt.0.0d0) .and.
     *     (dabs(fix_vol).lt.pr_lim) ) then
         go to 300
      end if
*     
*     Implement the default.
      if (fix_vol.le.0.0d0) then
         write(*,'(1a)')
     *        '        Adopting the value 5.0 for fix_vol.'
         fix_vol=5.0d0
      end if
*     
*     Define a handy constant.
      xim_max=fix_vol/3.0d0
*     
*     Acquire the value of the parameter \xi_{M,1}.
 400  write(*,'(1a,$)')
     *     'Enter the lower value of \xi_{M} (>0.0) [1.0E-9]:          '
      read(*,'(f43.36)',err=400) xim_var_1
*     
*     Prevention against a very small non-zero value.
      if ( (xim_var_1.gt.0.0d0) .and.
     *     (dabs(xim_var_1).lt.pr_lim) ) then
         go to 400
      end if
*     
*     Implement the default.
      if (xim_var_1.le.0.0d0) then
         write(*,'(1a)')
     *        '        Adopting the value 1.0E-9 for \xi_{M,1}.'
         xim_var_1=1.0d-9
      end if
*     
*     Acquire the value of the parameter \xi_{M,2}.
 500  write(*,'(1a,e18.11,1a,$)')
     *     'Enter the higher value of \xi_{M} < [',xim_max,']:  '
      read(*,'(f43.36)',err=500) xim_var_2
*     
*     Prevention against an illegal value.
      if (xim_var_2.ge.xim_max) then
         go to 500
      end if
*     
*     Prevention against a very small non-zero value.
      if ( (xim_var_2.gt.0.0d0) .and.
     *     (dabs(xim_var_2).lt.pr_lim) ) then
         go to 500
      end if
*     
*     Implement the default.
      if (xim_var_2.le.0.0d0) then
         write(*,'(1a,e18.11,1a)')
     *        '        Adopting the value',xim_max,' for \xi_{M,2}.'
         xim_var_2=xim_max
      end if
*     
*     Acquire the lower value of the parameter \xi_{1}.
 600  write(*,'(1a,$)')
     *     'Enter the lower value of the radius \xi_{1} (>0) [0.1]:    '
      read(*,'(f43.36)',err=600) xi_one_1
*     
*     Prevention against a very small non-zero value.
      if ( (xi_one_1.gt.0.0d0) .and.
     *     (dabs(xi_one_1).lt.pr_lim) ) then
         go to 600
      end if
*     
*     Implement the default.
      if (xi_one_1.le.0.0d0) then
         write(*,'(1a)')
     *        '        Adopting the lower value 0.1 for \xi_{1}.'
         xi_one_1=1.0d-1
      end if
*     
*     Acquire the upper value of the parameter \xi_{1}.
 700  write(*,'(1a,$)') 
     *     'Enter the upper value of the radius \xi_{1} (>0) [10.0]:   '
      read(*,'(f43.36)',err=700) xi_one_2
*     
*     Prevention against a very small non-zero value.
      if ( (xi_one_2.gt.0.0d0) .and.
     *     (dabs(xi_one_2).lt.pr_lim) ) then
         go to 700
      end if
*     
*     Implement the default.
      if (xi_one_2.le.1.0d0) then
         write(*,'(1a)')
     *        '        Adopting the upper value 10.0 for \xi_{1}.'
         xi_one_2=1.0d1
      end if
*     
*     Define the plotting variation of \xi_{1}.
      xi_del_var=(xi_one_2-xi_one_1)/dble(np_dat)
*     
*     The first point is known analytically.
*     
*     Store the value of \xi_{1}.
*     xi1variable(0)=0.0d0
*     
*     Store the value of \xi_{2}.
*     xi2variable(0)=dexp(dlog(fix_vol)/3.0d0)
*     
*     Store the value of the energy.
*     boundenergy(0)=xim_max
*     
*     Loop over the values of \xi_{1}.
      do ip_dat=1,np_dat+1 
*     
*     Determine the value of \xi_{1}.
         xi1_var=xi_one_1+xi_del_var*dble(ip_dat-1)
*     
*     Get the corresponding value of \xi_{M}.
         xienergy_var=
     *        xienergy_fun
     *        (ni_dat
     *        ,fix_vol
     *        ,xim_var_1
     *        ,xim_var_2
     *        ,xi1_var
     *        ,xi2_var
     *        ,lflg_error)
*     
*     Check for error before proceeding.
         if (lflg_error.ne.0) then
*     
*     If in error, write our an error message and stop.
            write(*,*)
            write(*,*) 'XIENERGY ERROR:'
            write(*,*) '       failure of xienergy_fun():'
            write(*,*) '       \xi_{M,1} = ',xim_var_1
            write(*,*) '       \xi_{M,2} = ',xim_var_2
            write(*,*) '       xienergy  = ',xienergy_var
            write(*,*)
*     
            stop
*     
         end if
*     
*     Write out an extra newline.
         write(*,*)
*     
*     Store the value of \xi_{1}.
         xi1variable(ip_dat)=xi1_var
*     
*     Store the value of \xi_{2}.
         xi2variable(ip_dat)=xi2_var
*     
*     Store the value of the energy.
         boundenergy(ip_dat)=xienergy_var
*     
      end do
*     
*     Open an output data file.
      open(unit=1,file='xienergy.dat')
*     
*     Write out the data as columns on a single file.
      do ip_dat=1,np_dat+1
         write(1,*)
     *        xi1variable(ip_dat)
     *        ,boundenergy(ip_dat)
      end do
*     
*     Close the data file.
      close(unit=1)
*     
      stop
*     
      END
