*     JLdL 02Jan21.
*     
*     Copyright (C) 2020-2021 by Jorge L. deLyra <delyra@lmail.if.usp.br>.
*     This program can be copied and/or distributed freely under the
*     GPL license; see the terms and conditions in the files within
*     the sub-directory doc/ of this source directory.
*     
*     A function to return the value of the mathematical formula
*     that appears on the left-hand side of the algebraic equation
*     shown in Equation (31) of the paper, as a function of \xi_{M};
*     the variables passed are:
*     
*     xi1_var_loc : the inner shell radius \xi_{1};
*     xi2_var_loc : the outer shell radius \xi_{2};
*     xim_var_loc : the Schwarzschild radius \xi_{M};
*     xi_del0_loc : the standard variation of \xi;
*     lflg_error  ; an error flag (output).
*     
*     This routine uses the following additional local
*     double-precision temporary variables:
*     
*     polynomial_loc : local temporary for the polynomial;
*     fintegral_loc  : local temporary for the integral;
*     equation_loc   : local temporary for the equation.
*     
*     Complex formulas contained in this routine: the algebraic equation.
*     
      FUNCTION EQUATION_FUN
     *     (xi1_var_loc
     *     ,xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_del0_loc
     *     ,lflg_error)
*     
*     Double precision everything; due to the gfortran compiler option
*     -fdefault-real-8, which is in use in the Makefile, this is actually
*     quadruple precision, in the traditional sense of the term, that is,
*     it is actually real*16 (128-bit real data).
      implicit double precision (a-h,o-z)
*     
*     Initialize the error flag.
      lflg_error=0
*     
*     Retrieve the value of the polynomial at \xi_{1}.
      polynomial_loc=
     *     polynomial_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi1_var_loc)
*     
*     Check for the strict positivity of the polynomial; note
*     that this verifies it on the whole integration interval;
*     if this does not succeed, then write out an error message,
*     raise the error flag and return prematurely in error.
      if ( (polynomial_loc.le.0.0d0)
     *     .or.
     *     (xi2_var_loc.le.xim_var_loc) ) then
         write(*,*)
         write(*,*) 'EQUATION_FUN ERROR:'
         write(*,*) '       cubic polynomial not strictly positive'
         write(*,*) '       at \xi_{1} and/or at \xi_{2}, where'
         write(*,*) '       \xi_{1} = ',xi1_var_loc
         write(*,*) '       \xi_{2} = ',xi2_var_loc
         write(*,*) '       \xi_{M} = ',xim_var_loc
         write(*,*) '       Therefore, the equation does not exist'
         write(*,*)
         lflg_error=1
         return
      end if
*     
*     Retrieve the value of the integral.
      fintegral_loc=
     *     fintegral_fun
     *     (xi1_var_loc
     *     ,xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_del0_loc
     *     ,lflg_error)
*     
*     Calculate the algebraic equation.
      equation_loc=0.0d0
      equation_loc=equation_loc
     *     +dsqrt(xi1_var_loc)/dsqrt(polynomial_loc)
      equation_loc=equation_loc
     *     -dsqrt(xi2_var_loc)/dsqrt(3.0d0*(xi2_var_loc-xim_var_loc))
      equation_loc=equation_loc
     *     +1.5d0*fintegral_loc
*     
*     Pass the function.
      equation_fun=equation_loc
*     
      return
*     
      END
