*     JLdL 02Jan21.
*     
*     Copyright (C) 2020-2021 by Jorge L. deLyra <delyra@lmail.if.usp.br>.
*     This program can be copied and/or distributed freely under the
*     GPL license; see the terms and conditions in the files within
*     the sub-directory doc/ of this source directory.
*     
*     A function to return the value of the integrand of the
*     integral that is shown in Equations (30) and (31) of
*     the paper; the variables passed are:
*     
*     xi2_var_loc : the outer shell radius \xi_{2};
*     xim_var_loc : the Schwarzschild radius \xi_{M};
*     xi_var_loc  : the radius variable \xi;
*     lflg_error  ; an error flag (output).
*     
*     This routine uses the following additional local
*     double-precision temporary variables:
*     
*     polynomial_loc : local temporary for the polynomial;
*     finteg_num_loc : local temporary for the numerator;
*     finter_den_loc : local temporary for the denominator;
*     fintegrand_loc : local temporary for the integrand.
*     
*     The integrand, with the polynomial in the denominator,
*     is given by (LaTeX codes):
*     
*     .                     \xi^{5/2}
*     -----------------------------------------------------
*     [ \xi_{2}^{3} - \xi^{3} + 3 ( \xi - \xi_{M} ) ]^{3/2}
*     
*     Complex formulas contained in this routine: the integrand.
*     
      FUNCTION FINTEGRAND_FUN
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_var_loc
     *     ,lflg_error)
*     
*     Double precision everything; due to the gfortran compiler option
*     -fdefault-real-8, which is in use in the Makefile, this is actually
*     quadruple precision, in the traditional sense of the term, that is,
*     it is actually real*16 (128-bit real data).
      implicit double precision (a-h,o-z)
*     
*     Initialize the error flag.
      lflg_error=0
*     
*     Calculate the function.
*     
*     Retrieve the value of the polynomial.
      polynomial_loc=
     *     polynomial_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_var_loc)
*     
*     Check the strict positivity condition; if this does not succeed,
*     then write out an error message, raise the error flag, zero off
*     the function, and return prematurely in error.
      if ( (polynomial_loc.le.0.0d0)
     *     .or.
     *     (xi2_var_loc.le.xim_var_loc) ) then
         write(*,*)
         write(*,*) 'FINTEGRAND_FUN ERROR:'
         write(*,*) '       cubic polynomial not strictly positive'
         write(*,*) '       at \xi and/or at \xi_{2}, where'
         write(*,*) '       \xi     = ',xi_var_loc
         write(*,*) '       \xi_{2} = ',xi2_var_loc
         write(*,*) '       \xi_{M} = ',xim_var_loc
         write(*,*) '       Therefore, the integrand does not exist'
         write(*,*)
         lflg_error=1
         fintegrand_loc=0.0d0
         fintegrand_fun=fintegrand_loc
         return
      end if
*     
*     Calculate the numerator.
      finteg_num_loc=dsqrt(xi_var_loc)**5
*     
*     Calculate the denominator.
      finter_den_loc=dsqrt(polynomial_loc)**3
*     
*     Calculate the ratio.
      fintegrand_loc=finteg_num_loc/finter_den_loc
*     
*     Pass the function.
      fintegrand_fun=fintegrand_loc
*     
      return
*     
      END
