*     JLdL 02Jan21.
*     
*     Copyright (C) 2020-2021 by Jorge L. deLyra <delyra@lmail.if.usp.br>.
*     This program can be copied and/or distributed freely under the
*     GPL license; see the terms and conditions in the files within
*     the sub-directory doc/ of this source directory.
*     
*     A function to return the value of the variable \xi_{M} related
*     to the total energy of the bound system, as a function of the
*     variable \xi_{1}, given constant values of the energy density
*     \rho_{0} and of the coordinate volume of the system; giving a
*     value of \rho_{0} is the same as giving a value of \Upsilon_{0}
*     and hence of r_{0}, which does not appear in the formulas in
*     terms of the dimensionless variables; once the value of r_{0}
*     is known, giving a value of the coordinate volume is the same
*     as giving a value of the dimensionless volume variable
*     
*     {\cal V} = \xi_{2}^{3} - \xi_{1}^{3},
*     
*     which is the ratio given by the coordinate volume of the system
*     divided by the volume of a sphere of radius r_{0}; therefore the
*     only variable which has do be given and kept fixed is {\cal V};
*     the variables passed are:
*     
*     ni_dat_loc    : the number of integration points;
*     fix_vol_loc   : the value of the fixed volume;
*     xim_var_1_loc : the global lower bound for \xi_{M};
*     xim_var_2_loc : the global upper bound for \xi_{M};
*     xi1_var_loc   : the value of \xi_{1};
*     xi2_var_loc   : the value of \xi_{2} (output);
*     lflg_error    ; an error flag (output).
*     
*     This routine uses the following additional local
*     double-precision temporary variables:
*     
*     equation_1_loc : local temporary for the equation lower bound;
*     equation_2_loc : local temporary for the equation upper bound;
*     xienergy_loc   : local temporary for the function;
*     xim_var_loc    : local temporary for the \xi_{M} variable;
*     xim_delta_loc  : local temporary for the \xi_{M} interval.
*     
*     Complex formulas contained in this routine: none.
*     
      FUNCTION XIENERGY_FUN
     *     (ni_dat_loc
     *     ,fix_vol_loc
     *     ,xim_var_1_loc
     *     ,xim_var_2_loc
     *     ,xi1_var_loc
     *     ,xi2_var_loc
     *     ,lflg_error)
*     
*     Double precision everything; due to the gfortran compiler option
*     -fdefault-real-8, which is in use in the Makefile, this is actually
*     quadruple precision, in the traditional sense of the term, that is,
*     it is actually real*16 (128-bit real data).
      implicit double precision (a-h,o-z)
*     
*     Include the global fixed parameters.
      include 'fp_def.f'
*     
*     Pass variables that will be changed to temporary local ones.
      xim_var_1_tmp=xim_var_1_loc
      xim_var_2_tmp=xim_var_2_loc
*     
*     Initialize the error flag.
      lflg_error=0
*     
*     Determine the value of \xi_{2}; that is, calculate
*     \xi_{2} from {\cat V} and \xi_{1}.
      xi2_var_loc=fix_vol_loc+xi1_var_loc**3
      xi2_var_loc=dexp(dlog(xi2_var_loc)/3.0d0)
*     
*     Having \xi_{2}, reset the value of the \xi_{M} upper bound
*     if necessary, so that \xi_{M} remains below \xi_{2}.
      if (xim_var_2_tmp.ge.xi2_var_loc) then
         xim_var_2_tmp=0.99d0*xi2_var_loc
      end if
*     
*     Define the standard integration variation of \xi.
      xi_del0_loc=(xi2_var_loc-xi1_var_loc)/dble(ni_dat_loc)
*     
*     Define the efective precision level for the exponential sandwich.
      pr_lev=1.0d-6/dble(ni_dat_loc)
*     
*     We must now determine \xi_{M}; the code here is just
*     like the one used in the program Solve_equation.
*     
*     Get the values of the formula at the lower bound.
      equation_1_loc=
     *     equation_fun
     *     (xi1_var_loc
     *     ,xi2_var_loc
     *     ,xim_var_1_tmp
     *     ,xi_del0_loc
     *     ,lflg_error)
*     
*     Check for error before proceeding; if in error, then write
*     out an error message, raise the error flag, zero off the
*     function, and return prematurely in error.
      if (lflg_error.ne.0) then
         write(*,*)
         write(*,*) 'XIENERGY_FUN ERROR:'
         write(*,*) '       bad initial \xi_{M} lower bound leading'
         write(*,*) '       to failure of equation_fun():'
         write(*,*) '       \xi_{M,1}  = ',xim_var_1_tmp
         write(*,*) '       equation_1 = ',equation_1_loc
         write(*,*)
         lflg_error=1
         xienergy_loc=0.0d0
         xienergy_fun=xienergy_loc
         return
      end if
*     
*     Get the values of the formula at the upper bound.
      equation_2_loc=
     *     equation_fun
     *     (xi1_var_loc
     *     ,xi2_var_loc
     *     ,xim_var_2_tmp
     *     ,xi_del0_loc
     *     ,lflg_error)
*     
*     Check for error before proceeding; if in error, then write
*     out an error message, raise the error flag, zero off the
*     function, and return prematurely in error.
      if (lflg_error.ne.0) then
         write(*,*)
         write(*,*) 'XIENERGY_FUN ERROR:'
         write(*,*) '       bad initial \xi_{M} upper bound leading'
         write(*,*) '       to failure of equation_fun():'
         write(*,*) '       \xi_{M,2}  = ',xim_var_2_tmp
         write(*,*) '       equation_2 = ',equation_2_loc
         write(*,*)
         lflg_error=1
         xienergy_loc=0.0d0
         xienergy_fun=xienergy_loc
         return
      end if
*     
*     Determine the initial situation.
*     
*     Check for an accidental hit at the left end.
      if (equation_1_loc.eq.0.0d0) then
*     
*     Get the solution and return.
         xienergy_loc=xim_var_1_tmp
*     
*     Pass the function.
         xienergy_fun=xienergy_loc
*     
         return
*     
*     Check for an accidental hit at the right end.
      else if (equation_2_loc.eq.0.0d0) then
*     
*     Get the solution and return.
         xienergy_loc=xim_var_2_tmp
*     
*     Pass the function.
         xienergy_fun=xienergy_loc
*     
         return
*     
*     Check the consistency of the two bounds; if this does not
*     succeed, then write out an error message, raise the error
*     flag, zero off the function, and return prematurely in error.
      else if (equation_1_loc*equation_2_loc.gt.0.0d0) then
         write(*,*)
         write(*,*) 'XIENERGY_FUN ERROR:'
         write(*,*) '       bad initial \xi_{M} bounds:'
         write(*,*) '       \xi_{M,1}  = ',xim_var_1_tmp
         write(*,*) '       equation_1 = ',equation_1_loc
         write(*,*) '       \xi_{M,2}  = ',xim_var_2_tmp
         write(*,*) '       equation_2 = ',equation_2_loc
         write(*,*)
         lflg_error=1
         xienergy_loc=0.0d0
         xienergy_fun=xienergy_loc
         return
      end if
*     
*     Solve for the root.
*     
*     Format for the monitoring data.
 10   format('\xi_1 = ',f8.4
     *     ,'    loops = ',i3
     *     ,'    delta = ',g24.16
     *     ,'    \xi_M = ',g24.16,$)
*     
*     Turn off the xterm wraparound mode.
      write(*,'(''[?7;l'',$)')
*     
*     Initialize a counter.
      ic_dat=0
*     
*     Calculate the \xi_{M} interval.
      xim_delta_loc=xim_var_2_tmp-xim_var_1_tmp
*     
*     Write out some progress report.
*     
*     Save the cursor position.
      write(*,'(''7'',$)')
*     
*     Write the monitoring data to the terminal.
      write(*,10) xi1_var_loc,ic_dat,xim_delta_loc,0.0d0
*     
*     Restore the cursor position.
      write(*,'(''8'',$)')
*     
*     Loop until an acceptable precision is reached.
      do while (xim_delta_loc.gt.pr_lev)
*     
*     Calculate the middle point.
         ic_dat=ic_dat+1
         xim_var_m_loc=0.5d0*(xim_var_1_tmp+xim_var_2_tmp)
         xienergy_loc=
     *        equation_fun
     *        (xi1_var_loc
     *        ,xi2_var_loc
     *        ,xim_var_m_loc
     *        ,xi_del0_loc
     *        ,lflg_error)
*     
*     Check for error before proceeding; if in error, then write
*     out an error message, raise the error flag, zero off the
*     function, and return prematurely in error.
         if (lflg_error.ne.0) then
            write(*,*)
            write(*,*) 'XIENERGY_FUN ERROR:'
            write(*,*) '       bad value of \xi_{M} leading'
            write(*,*) '       to failure of equation_fun():'
            write(*,*) '       \xi_{M}  = ',xim_var_m_loc
            write(*,*) '       xienergy = ',xienergy_loc
            write(*,*)
            lflg_error=1
            xienergy_loc=0.0d0
            xienergy_fun=xienergy_loc
            return
         end if
*     
*     Check for an accidental hit at the middle point.
         if (xienergy_loc.eq.0.0d0) then
*     
*     Get the solution and return.
            xienergy_loc=xim_var_m_loc
*     
*     Pass the function.
            xienergy_fun=xienergy_loc
*     
            return
*     
*     Compare the sign with the left end.
         else if (xienergy_loc*equation_1_loc.lt.0.0d0) then
*     
*     Update the variables accordingly.
            xim_var_2_tmp=xim_var_m_loc
            equation_2_loc=xienergy_loc
*     
*     Compare the sign with the right end.
         else if (xienergy_loc*equation_2_loc.lt.0.0d0) then
*     
*     Update the variables accordingly.
            xim_var_1_tmp=xim_var_m_loc
            equation_1_loc=xienergy_loc
*     
         end if
*     Update the \xi_{M} interval.
         xim_delta_loc=xim_var_2_tmp-xim_var_1_tmp
*     
*     Write out some progress report.
*     
*     Save the cursor position.
         write(*,'(''7'',$)')
*     
*     Write the monitoring data to the terminal.
         write(*,10) xi1_var_loc,ic_dat,xim_delta_loc,xim_var_m_loc
*     
*     Restore the cursor position.
         write(*,'(''8'',$)')
*     
      end do
*     
*     Turn the xterm wraparound mode back on.
      write(*,'(''[?7;h'',$)')
*     
*     Get the solution and return.
      xienergy_loc=xim_var_m_loc
*     
*     Pass the function.
      xienergy_fun=xienergy_loc
*     
      return
*     
      END
